;==============================================================
; Inicializar VDP
;==============================================================

; Call DefaultInitialiseVDP to set up VDP to default values.
; Also defines NameTableAddress, SpriteTableAddress and SpriteSet
; which can be used elsewhere.
; To change the values used, copy and paste the modified data
; and code into the main source. Data is commented to help.
;==============================================================
.section "Inicializae VDP" free
VDPInit:
    push hl
    push bc
        ld hl,_Data
        ld b,_End-_Data
        ld c,$bf
        otir
    pop bc
    pop hl
    ret

.define SpriteSet           0       ; 0 for sprites to use tiles 0-255, 1 for 256+
.define NameTableAddress    $3800   ; must be a multiple of $800; usually $3800; fills $700 bytes (unstretched)
.define SpriteTableAddress  $3f00   ; must be a multiple of $100; usually $3f00; fills $100 bytes

_Data:
    .db %00000100,$80
    ;    |||||||`- Disable synch
    ;    ||||||`-- Enable extra height modes
    ;    |||||`--- SMS mode instead of SG
    ;    ||||`---- Shift sprites left 8 pixels
    ;    |||`----- Enable line interrupts
    ;    ||`------ Blank leftmost column for scrolling
    ;    |`------- Fix top 2 rows during horizontal scrolling
    ;    `-------- Fix right 8 columns during vertical scrolling
    .db %10000100,$81
    ;     |||| |`- Zoomed sprites -> 16x16 pixels
    ;     |||| `-- Doubled sprites -> 2 tiles per sprite, 8x16
    ;     |||`---- 30 row/240 line mode
    ;     ||`----- 28 row/224 line mode
    ;     |`------ Enable VBlank interrupts
    ;     `------- Enable display
    .db (NameTableAddress>>10) |%11110001,$82
    .db (SpriteTableAddress>>7)|%10000001,$85
    .db (SpriteSet<<2)         |%11111011,$86
    .db $f|$f0,$87
    ;    `-------- Border palette colour (sprite palette)
    .db $00,$88
    ;    ``------- Horizontal scroll
    .db $00,$89
    ;    ``------- Vertical scroll
    .db $ff,$8a
    ;    ``------- Line interrupt spacing ($ff to disable)
_End:
.ends

;==============================================================
; Inicializar VRAM
;==============================================================
.section "Clear VRAM" free

VRAMInit:
  push af
  push hl
    ld hl,$0000
    call VRAMToHL
    ; 2. Output 16KB of zeroes
    ld hl, $4000    ; Counter for 16KB of VRAM
    ld a,$00        ; Value to write
-:
    out ($be),a ; Output to VRAM, auto-incremented
    dec h
    jp nz,-
    dec l
    jp nz,-
  pop hl
  pop af
ret
.ends
;==============================================================
; VRAM to HL
;==============================================================
.section "VRAM to HL"
VRAMToHL:
  push af      	;salvar A en la pila
    ld a,l
    out ($bf),a
    ld a,h
    or $40
    out ($bf),a
  pop af
ret
.ends

;==============================================================
; Cargar paleta
;==============================================================
; hl = location
; b  = number of values to write
; c  = palette index to start at (<32)
;==============================================================
.section "Load palette" free
LoadPalette:
  push af
  push bc
  push hl
    ld a,c
    out ($bf),a         ;Index
    ld a,$c0
    out ($bf),a         ;Palette write identifier
    ld c,$be		;Escribir a $be
    otir		;Output b bytes starting at hl to port c
  pop hl
  pop bc
  pop af
ret
.ends


;==============================================================
; Cargar tiles
;==============================================================
; Parameters:
; hl = tile number to start at
; ix = location of tile data
; bc = No. of tiles to load
; d  = bits per pixel
;==============================================================
.section "Tile loader" FREE
LoadTiles:
    push af
    push bc
    push de
    push hl
    push ix
        ; Tell VDP where I want to write (hl<<5)
        sla l
        rl h
        sla l
        rl h
        sla l
        rl h
        sla l
        rl h
        sla l
        rl h
        ld a,l
        out ($bf),a
        ld a,h
        or $40
        out ($bf),a

        ; I need to output bc*8 bytes so I need to modify bc (<<3)
        sla c
        rl b
        sla c
        rl b
        sla c
        rl b

        ; Write data
        _Loop:
            ; Restore counter
            ld e,d

            _DataLoop:
                ; Write byte
                ld a,(ix+0)
                out ($be),a
                dec e
                inc ix
                jp nz,_DataLoop

            ; Restore counter
            ld e,d
            _BlankLoop:
                ; Write blank data to fill up the rest of the tile definition
                inc e
                ld a,e
                cp 5
                jp z,_NoMoreBlanks

                ld a,0
                out ($be),a
                jp _BlankLoop

            _NoMoreBlanks:

            dec bc
            ld a,b
            or c
            jp nz,_Loop

    pop ix
    pop hl
    pop de
    pop bc
    pop af
    ret
.ends

;==============================================================
; V Counter reader
; Waits for 2 consecutive identical values (to avoid garbage)
; Returns in a *and* b
;==============================================================
.section "Get VCount" free
GetVCount:
    in a,($7e)  ; get VCount
  -:ld b,a      ; store it
    in a,($7e)  ; and again
    cp b        ; Is it the same?
    jp nz,-     ; If not, repeat
    ret         ; If so, return it in a (and b)
.ends

.section "Wait for VBlank without interrupts" free
WaitForVBlankNoInt:
    push bc
    push af
      -:call GetVCount
;        cp 192       ;if usding this, youi have more time of Vblank
                      ;but it runs at different speeds in PAL / NTSC
        cp 243
        jp nz,-
    pop af
    pop bc
    ret
.ends

;==============================================================
.section "Detect domestic/export console"
DetectMode:
  push af
    in a,($3F)
  push af

    ld a,%11110101	;output TH pins=1
    out ($3F),a
    in a,($DD)		;check if they are 1
    and %11000000
    jp z,+		;if not, is Japanese


    ld a,%01010101	;output 0s now
    out ($3F),a
    in a,($DD)
    and %11000000	;if any is read = 1
    jp nz,+		;then is Japanese

    ld a,0
    ld (DomesticMode),a
    jp ++

    +:
    ld a,1

    ++:
    ld (DomesticMode),a
  pop af
    out ($3F),a
  pop af
ret

;==============================================================
DetectTVType:
;  MODIFICA el registro a, h,l,d,e
 ; Returns a=0 for NTSC, a=1 for PAL
 ; uses a, hl, de
    di             ; disable interrupts
    ld a,%01100000 ; set VDP such that the screen is on
    out ($bf),a    ; with VBlank interrupts enabled
    ld a,$81
    out ($bf),a
    ld hl,$0000    ; init counter
  -:in a,($bf)     ; get VDP status
    or a           ; inspect
    jp p,-         ; loop until frame interrupt flag is set
  -:in a,($bf)     ; do the same again, in case we were unlucky and came in just
    or a           ;   before the start of the VBlank with the flag already set
    jp p,-
    ; the VDP must now be at the start of the VBlank
  -:inc hl         ; (6 cycles) increment counter until interrupt flag comes on again
    in a,($bf)     ; (11 cycles)
    or a           ; (4 cycles)
    jp p,-         ; (10 cycles)
    xor a          ; reset carry flag, also set a=0
    ld de,2048     ; see if hl is more or less than 2048
    sbc hl,de
    ret c          ; if less, return a=0
    ld a,1
    ret            ; if more or equal, return a=1
;==============================================================
.ends


